#!/usr/bin/perl -w 
#
# Author: Andrew Robert
# Date:   06/07/2005
# Program: dump_messages.pl
#
# Version Date       Programmer    Description
# 1.0                              Initial Release
# 1.1     6/20/05    AAR           Add functionality to prevent automatic upcase of command-line arguments
# 1.2     6/20/05    AAR           Add functionality to specify how many messages to dump
# 1.3     6/21/05    AAR           Modify -c function to not upcase only if set.
# 1.4     6/21/05    AAR           Add functionality to force overwrite of the output file
# 1.5     6/27/05    AAR           Modify -r and -c switchs so that they do not require arguments

use strict;
use MQSeries;
use MQSeries::Queue;
use MQSeries::Message;
use Getopt::Std;


my $sourceq    = "";
my $qmgr       = "";
my $outfile    = "";
my $status     = "";
my $expression = "";
my $count      = 0;


#
# Read and assign input from command line 
#
my %opts;
getopts("m:q:f:n:rc", \%opts);



#
# Test number of command line arguments. Exit and 
# supply usage summary if not enough were entered.
# If enough are available, capitilize the queue and queue manager
# in case they were entered in lower case.
#
if  (  keys(%opts) < 3 )
   {
     print "\nMissing command line parameters\n";
     print "Usage:   dump_messages.pl -m {queue manager} -q {queue} -f {output file}\n";
     print "Optional -c disable automatic case change\n"; 
     print "Optional -r force overwrite output file\n"; 
     print "Optional -n #  to specify number of messages to extract\n\n";
     exit;
   }


#
# Test case option. If set, disable automatic upper casing of command line arguments.
# Default behavior is to upper case the queue and queue manager.
#
if ( defined $opts{c} )
  {  
   $sourceq   = $opts{q};  
   $qmgr      = $opts{m};
   $outfile   = $opts{f};
  }
else
  {
   $sourceq   = uc( $opts{q} );  
   $qmgr      = uc( $opts{m} );
   $outfile   = lc( $opts{f} );
  }


   
# Create source queue object 
# linking queue manager and destination queue
#
my $sourcequeue = MQSeries::Queue->new
    (
     QueueManager       => $qmgr,
     Queue              => $sourceq,
     Mode               => 'input'
    )|| die ("Unable to instantiate $qmgr::$sourceq object\n"); 



#
# Check if output file exists. If it does, exit to prevent accidental overwrite of file.
# Otherwise, attempt to open the output file for write. 
#    
if ( -e $outfile && ( not defined $opts{r} ) )
   {
    die( "\nFile $outfile exists.. Aborting to prevent over-write\n\nPlease select another file name.\n\n");
   }


#
# Attempt to open output file for write
#
open(FD, ">", $outfile)  or die("Couldn't open $outfile : $!\n") ; 

#
# Retrieve messages from source queue and write them to a file
#
print "\n$expression\n";

while (1) 
      {
       #
       # Clear message buffer and retrieve new message if possible
       # 
       my $returned_message = MQSeries::Message->new( Data =>"" );
       my $rc=$sourcequeue->Get( Message => $returned_message);

       
       if ($rc == 1) # A code of 1 means that a message was received 
          {
           #
           # Modify contents of message data to ensure contents 
           # do not conflict with tags
           # 
           my $msg_data = $returned_message->Data();
           $count++;

           print "Writing message $count\n";
           print FD "<STARTMESSAGE>";
               
           #
           # Iterate over keys, modify returned value, and convert it.
           # 
           my $msg_header = $returned_message->MsgDesc();
           $msg_header->{Data}=$msg_data;
                
           foreach (keys %$msg_header)
                   {
                    # Assign key value to string
                    my $arg  = "$_:$msg_header->{$_}";  
                       
                    #
                    # encode the data 
                    #
                    $arg=~s/\\/\\\\/g;
                    $arg=~s/\n/\\n/g;
                    
                    # 
                    # If the encoded line is blank, go to next line
                    # 
                    if  ( $arg=~/^\s*$/  )
                        {
                         next;
                        }
                    #
                    # Write line to data file
                    #
                    print FD "\n$arg";
                   }
            print FD "\n<ENDMESSAGE>\n";
            
            #
            # Test number of messages processed if a limit was set at the command line
            #
            if ( defined $opts{n} )
               {
                if ( $opts{n}  < ($count +1))
                   {
                    last;
                   }    
               }
               
           }
        elsif ($rc == -1) # a code of -1 means that no message are available
           {
            my $status="\nNo more messages found on $sourceq\n \n"; 
            last;       
           }
        else     
           {
            my $status="\nError retrieving messages from $sourceq." .
            "\nCompCode =" . $sourcequeue->CompCode() . 
            "\nReason   =" . $sourcequeue->Reason() . "\n";
            last;       
           }
       }

#
# Close previously opened file
#
close(FD);
print "\n$status\n";
